from ehelply_bootstrapper.integrations.integration import Integration
from ehelply_bootstrapper.utils.state import State
from datetime import datetime
from typing import Union
from pydantic import BaseModel
from ehelply_microservice_library.integrations.fact import get_fact_endpoint


class BasicMeta(BaseModel):
    """
    Basic meta
    """
    name: str = None
    slug: str = None


class DetailedMeta(BaseModel):
    """
    Detailed meta based on Notes
    """
    summary_uuid: str = None
    description_uuid: str = None


class DatesMeta(BaseModel):
    """
    Date based meta
    """
    created_at: str = datetime.utcnow().isoformat()
    updated_at: str = datetime.utcnow().isoformat()


class MetaBase(BaseModel):
    """
    Meta
    """
    basic: BasicMeta = BasicMeta()
    detailed: DetailedMeta = DetailedMeta()
    custom: dict = {}
    dates: DatesMeta = DatesMeta()


class MetaDynamo(MetaBase):
    """
    A meta from DynamoDB
    """
    uuid: str


class BasicMetaCreate(BaseModel):
    """
    Basic meta
    """
    name: str = None
    slug: bool = True


class DetailedMetaCreate(BaseModel):
    """
    Detailed meta based on Notes
    """
    summary: str = None
    description: str = None


class DetailedMetaGet(DetailedMetaCreate):
    """
    Detailed meta based on Notes
    """
    summary_history: list = None
    description_history: list = None


class MetaCreate(BaseModel):
    """
    Meta
    """
    basic: BasicMetaCreate = BasicMetaCreate()
    detailed: DetailedMetaCreate = DetailedMetaCreate()
    custom: dict = {}


class MetaGet(BaseModel):
    """
    Meta
    """
    uuid: str
    basic: BasicMeta = BasicMeta()
    detailed: Union[DetailedMetaGet, None] = DetailedMetaGet()
    custom: Union[dict, None] = {}
    dates: Union[DatesMeta, None] = DatesMeta()


class MetaSlugger(BaseModel):
    """
    Meta slugger
    """
    name: str


class Meta(Integration):
    """
    Meta integration is used to talk to the ehelply-meta microservice
    """

    def __init__(self) -> None:
        super().__init__("meta")

        self.m2m = State.integrations.get("m2m")

    def init(self):
        super().init()

    def load(self):
        super().load()

    def get_base_url(self) -> str:
        return get_fact_endpoint('ehelply-meta') + "/meta"

    def create(self, service: str, type_str: str, entity_uuid: str, meta: MetaCreate) -> Union[MetaDynamo, None, bool]:
        """
        Create new meta
        :param service:
        :param type_str:
        :param entity_uuid:
        :param meta:
        :return:
        """
        if type(service) is not str or type(type_str) is not str or type(entity_uuid) is not str:
            State.logger.warning('Invalid payload when trying to create meta.')
            return False

        data: dict = {
            "meta": meta.dict()
        }

        response = self.m2m.requests.post(
            self.get_base_url() + "/service/" + service + "/type/" + type_str + "/entity/" + entity_uuid,
            json=data)

        if response.status_code != 200:
            return None

        return MetaDynamo(**response.json())

    def get(self, service: str, type_str: str, entity_uuid: str, detailed: bool = False, custom: bool = False,
            dates: bool = False) -> Union[None, bool, MetaGet]:
        """
        Get meta
        :param service:
        :param type_str:
        :param entity_uuid:
        :param detailed:
        :param custom:
        :param dates:
        :return:
        """
        if type(service) is not str or type(type_str) is not str or type(entity_uuid) is not str or type(
                detailed) is not bool or type(custom) is not bool or type(dates) is not bool:
            State.logger.warning('Invalid payload when trying to get meta.')
            return False

        response = self.m2m.requests.get(
            self.get_base_url() + "/service/" + service + "/type/" + type_str + "/entity/" + entity_uuid,
            params={"detailed": detailed, "custom": custom, "dates": dates})

        if response.status_code != 200:
            return None

        return MetaGet(**response.json())

    def get_using_uuid(self, meta_uuid: str, detailed: bool = False, custom: bool = False, dates: bool = False) -> \
    Union[None, bool, MetaGet]:
        """
        Get meta
        :param meta_uuid:
        :param detailed:
        :param custom:
        :param dates:
        :return:
        """
        if type(meta_uuid) is not str or type(detailed) is not bool or type(custom) is not bool or type(
                dates) is not bool:
            State.logger.warning('Invalid payload when trying to get meta.')
            return False

        response = self.m2m.requests.get(self.get_base_url() + "/" + meta_uuid,
                                         params={"detailed": detailed, "custom": custom, "dates": dates})

        if response.status_code != 200:
            return None

        return MetaGet(**response.json())

    def update(self, service: str, type_str: str, entity_uuid: str, meta: MetaCreate) -> Union[MetaDynamo, None, bool]:
        """
        Update meta
        :param service:
        :param type_str:
        :param entity_uuid:
        :param meta:
        :return:
        """
        if type(service) is not str or type(type_str) is not str or type(entity_uuid) is not str:
            State.logger.warning('Invalid payload when trying to update meta.')
            return False

        data: dict = {
            "meta": meta.dict()
        }

        response = self.m2m.requests.put(
            self.get_base_url() + "/service/" + service + "/type/" + type_str + "/entity/" + entity_uuid,
            json=data)

        if response.status_code != 200:
            return None

        return MetaDynamo(**response.json())

    def update_using_uuid(self, meta_uuid: str, meta: MetaCreate) -> Union[MetaDynamo, None, bool]:
        """
        Update meta
        :param meta_uuid:
        :param meta:
        :return:
        """
        if type(meta_uuid) is not str:
            State.logger.warning('Invalid payload when trying to update meta.')
            return False

        data: dict = {
            "meta": meta.dict()
        }

        response = self.m2m.requests.put(
            self.get_base_url() + "/" + meta_uuid,
            json=data)

        if response.status_code != 200:
            return None

        return MetaDynamo(**response.json())

    def delete(self, service: str, type_str: str, entity_uuid: str):
        """
        Delete meta
        :param service:
        :param type_str:
        :param entity_uuid:
        :return:
        """
        if type(service) is not str or type(type_str) is not str or type(entity_uuid) is not str:
            State.logger.warning('Invalid payload when trying to delete meta.')
            return False

        response = self.m2m.requests.delete(
            self.get_base_url() + "/service/" + service + "/type/" + type_str + "/entity/" + entity_uuid)

        if response.status_code != 200:
            return None

        return response.json()

    def delete_using_uuid(self, meta_uuid: str):
        """
        Delete meta
        :param meta_uuid:
        :return:
        """
        if type(meta_uuid) is not str:
            State.logger.warning('Invalid payload when trying to delete meta.')
            return False

        response = self.m2m.requests.delete(self.get_base_url() + "/" + meta_uuid)

        if response.status_code != 200:
            return None

        return response.json()

    def touch(self, service: str, type_str: str, entity_uuid: str) -> Union[MetaDynamo, None, bool]:
        """
        Touch meta
        :param service:
        :param type_str:
        :param entity_uuid:
        :return:
        """
        if type(service) is not str or type(type_str) is not str or type(entity_uuid) is not str:
            State.logger.warning('Invalid payload when trying to touch meta.')
            return False

        response = self.m2m.requests.post(
            self.get_base_url() + "/service/" + service + "/type/" + type_str + "/entity/" + entity_uuid + "/touch")

        if response.status_code != 200:
            return None

        return MetaDynamo(**response.json())

    def slug(self, name: str):
        """
        Turn a string into a slug
        :param name:
        :return:
        """
        return self.m2m.requests.post(self.get_base_url() + "/slug", json={"name": name}).json()



from typing import List

import typer

cli = typer.Typer()


# @cli.command()
# def install():
#     print("Not implemented")
#
#
# @cli.command()
# def seed():
#     print("Not implemented")

@cli.command()
def export_code_docs():
    """Script to export the microservice code as HTML docs. Exports to /docs"""
    from pathlib import Path

    from ehelply_bootstrapper.utils.docs import export_code_docs as lib_export_code_docs

    from ehelply_microservice_library.cli.cli_state import CLIState

    class DocsService(CLIState.service):
        def if_dev_launch_dev_server(self) -> bool:
            return False

        def is_load_service_routers(self) -> bool:
            return False

    service = DocsService()

    lib_export_code_docs(
        root_path=Path(service.get_service_package_path()).resolve(),
        modules=["src"]
    )


@cli.command()
def export_api_docs(
    include_service_routers: bool = typer.Option(False, help="Include service routers in the docs"),
    include_ehelply_header: bool = typer.Option(True, help="Include eHelply header on the docs page"),
):
    """Script to export the ReDoc documentation page into a standalone HTML file. Exports to /api-docs"""

    with typer.progressbar(length=8, label="Exporting") as progress:

        typer.echo("Starting doc exports. Expect to see lots of bootstrap text in the log")

        import json

        import sys

        from pathlib import Path

        from datetime import datetime

        from ehelply_microservice_library.cli.cli_state import CLIState

        class DocsService(CLIState.service):
            def if_dev_launch_dev_server(self) -> bool:
                return False

            def is_load_service_routers(self) -> bool:
                return include_service_routers

        progress.update(1)

        service = DocsService()

        progress.update(3)

        if service.fastapi_driver:
            app = service.fastapi_driver.instance

            required_integrations: str = ""
            for integration in service.get_service_required_integrations():
                integration_name = integration.replace("-", " ").capitalize().replace("Ehelply", "eHelply")
                required_integrations += '<li><a href="https://github.com/eHelply/{integration}">{integration_name}</a></li>'.format(integration=integration, integration_name=integration_name)
                
            optional_integrations: str = ""
            for integration in service.get_service_optional_integrations():
                integration_name = integration.replace("-", " ").capitalize().replace("Ehelply", "eHelply")
                optional_integrations += '<li><a href="https://github.com/eHelply/{integration}">{integration_name}</a></li>'.format(integration=integration, integration_name=integration_name)

            if include_ehelply_header:
                HTML_TEMPLATE = """<!DOCTYPE html>
<html>
<head>
    <meta http-equiv="content-type" content="text/html; charset=UTF-8">
    <title>{service_name} {service_version} - Docs</title>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel="shortcut icon" href="{ehelply_logo_url}">
    <style>
        body {{
            margin: 0;
            padding: 0;
        }}
    </style>
    <style data-styled="" data-styled-version="4.4.1"></style>
</head>
<body>

    <div style='padding:25px;'>

        <h3>eHelply Microservice Documentation</h3>
        <a href="https://github.com/eHelply/docs-ehelply-microservices">Documentation Repository</a>

        <h4>{service_name} - {service_version}</h4>
        <a href="https://github.com/eHelply/docs-ehelply-microservices/{service_key}">{service_name} Documentation Revisions</a>

        <h5>Required Integrations</h5>
        <ul>
        {required_integrations}
        </ul>
        
        <h5>Optional Integrations</h5>
        <ul>
        {optional_integrations}
        </ul>
    </div>

    <hr>

    <div id="redoc-container"></div>
    <script src="https://cdn.jsdelivr.net/npm/redoc/bundles/redoc.standalone.js"> </script>
    <script>
        var spec = %s;
        Redoc.init(spec, {{}}, document.getElementById("redoc-container"));
    </script>
</body>
</html>
                """.format(
                    service_name=service.service_meta.name,
                    service_key=service.service_meta.key,
                    service_version=service.service_meta.version,
                    required_integrations=required_integrations,
                    optional_integrations=optional_integrations,
                    ehelply_logo_url="https://assets.ehelply.com/logo/ehelply/base/symbol_transparent_white.png",
                )

            else:
                HTML_TEMPLATE = """<!DOCTYPE html>
<html>
<head>
    <meta http-equiv="content-type" content="text/html; charset=UTF-8">
    <title>{service_name} {service_version} - Docs</title>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel="shortcut icon" href="{ehelply_logo_url}">
    <style>
        body {{
            margin: 0;
            padding: 0;
        }}
    </style>
    <style data-styled="" data-styled-version="4.4.1"></style>
</head>
<body>
    <div id="redoc-container"></div>
    <script src="https://cdn.jsdelivr.net/npm/redoc/bundles/redoc.standalone.js"> </script>
    <script>
        var spec = %s;
        Redoc.init(spec, {{}}, document.getElementById("redoc-container"));
    </script>
</body>
</html>
                """.format(
                    service_name=service.service_meta.name,
                    service_version=service.service_meta.version,
                    ehelply_logo_url="https://assets.ehelply.com/logo/ehelply/base/symbol_transparent_white.png",
                )

            docs_file: str = datetime.utcnow().strftime(
                "api-docs_" + service.service_meta.version + "_%Y%m%d-%H%M%S-utc.html")

            docs_location = Path(service.get_service_package_path()).resolve().joinpath('api-docs')
            docs_location.mkdir(exist_ok=True)

            progress.update(1)

            with open(docs_location.joinpath(docs_file), "w") as fd:
                print(HTML_TEMPLATE % json.dumps(app.openapi()), file=fd)

            progress.update(3)

        typer.echo("Doc export complete. Please check the api-docs folder.")

        sys.exit()


@cli.command()
def export_api_spec(
    include_service_routers: bool = typer.Option(False, help="Include service routers in the spec"),
):
    """Script to export Open API JSON spec to a file. Exports to /api-specs"""

    with typer.progressbar(length=8, label="Exporting") as progress:
        typer.echo("Starting spec export. Expect to see lots of bootstrap text in the log")

        import json

        import sys

        from pathlib import Path

        from datetime import datetime

        from ehelply_microservice_library.cli.cli_state import CLIState

        class DocsService(CLIState.service):
            def if_dev_launch_dev_server(self) -> bool:
                return False

            def is_load_service_routers(self) -> bool:
                return include_service_routers

        progress.update(1)

        service = DocsService()

        progress.update(2)

        if service.fastapi_driver:
            app = service.fastapi_driver.instance

            docs_file: str = datetime.utcnow().strftime(
                "api-spec_" + service.service_meta.version + "_%Y%m%d-%H%M%S-utc.json")

            docs_location = Path(service.get_service_package_path()).resolve().joinpath('api-specs')
            docs_location.mkdir(exist_ok=True)

            progress.update(1)

            data: dict = {
                "meta": service.service_meta.dict(),
                "spec": app.openapi()
            }

            progress.update(3)

            with open(docs_location.joinpath(docs_file), "w") as fd:
                json.dump(data, fd)

            progress.update(1)

        typer.echo("Doc export complete. Please check the api-specs folder.")

        sys.exit()


if __name__ == '__main__':
    cli()
